/**
    problema Concert
    clasa a 9-a
    autor: mrd. Razvan-Daniel Zoltan

    Complexitate: O(T * N * M), optimizat cu calupuri de 32 de biti
    Punctaj estimat: 40p
**/

#include <fstream>

using namespace std;
using ull = unsigned long long;

ifstream fin("concert.in");
ofstream fout("concert.out");

const int NMAX = 2000;
const int CHUNK_SIZE = 32;
const int MMAX = NMAX / CHUNK_SIZE;

const int allBitsSet = ~0;

int rows, columns, numberOfUpdates, numberOfQueries;

// dimensiunea matricei se transforma din N pe M in N pe M/64
// vom imparti numarul de coloane in calupuri de cate 64 de biti (exact cat este alocat pentru un unsigned long long)
int matrix[NMAX + 1][MMAX + 1];

void read() {
    fin >> rows >> columns;
    int value, chunk, chunkBit;
    for (int i = 1; i <= rows; ++i) {
        for (int j = 1; j <= columns; ++j) {
            fin >> value;
            if (value == 1) {
                // calculam din ce calup (chunk - coloana din matricea noua) face parte coloana initiala
                chunk = (j - 1) / CHUNK_SIZE + 1;
                // calculam bitul pe care trebuie sa il activam in calupul gasit
                chunkBit = (j - 1) % CHUNK_SIZE;
                // activam bitul gasit mai sus
                matrix[i][chunk] |= (1 << chunkBit);
            }
        }
    }
}

void update() {
    fin >> numberOfUpdates;
    int x1, y1, x2, y2;

    int currentColumn, currentChunk, endChunk;
    int maskStartBit, maskEndBit;
    ull maskStart, maskEnd;
    while (numberOfUpdates--) {
        fin >> x1 >> y1 >> x2 >> y2;
        endChunk = (y2 - 1) / CHUNK_SIZE + 1;
        for (int i = x1; i <= x2; ++i) {
            currentColumn = y1;
            while (currentColumn <= y2) {
                // calculam din ce calup (chunk - coloana din matricea noua) face parte coloana curenta
                currentChunk = (currentColumn - 1) / CHUNK_SIZE + 1;
                // calculam bitul de unde incepem sa activam biti in calupul gasit
                maskStartBit = (currentColumn - 1) % CHUNK_SIZE;
                // vom incerca sa obtinem o masca de biti care contine doar 1 intre bitii
                // de pozitie maskStartBit si maskEndBit
                if (currentChunk == endChunk) {
                    // suntem la ultimul calup in care o sa activam biti - acolo se afla si coloana y2
                    maskEndBit = (y2 - 1) % CHUNK_SIZE;
                    if (maskEndBit == CHUNK_SIZE - 1) {
                        // maskStart o sa contina biti de 1 de la inceput si pana la bitul (maskStartBit - 1)
                        maskStart = (1 << maskStartBit) - 1;
                        maskEnd = allBitsSet;
                    }
                    else {
                        // maskStart o sa contina biti de 1 de la inceput si pana la bitul (maskStartBit - 1)
                        maskStart = (1 << maskStartBit) - 1;
                        maskEnd = (1 << (maskEndBit + 1)) - 1;
                    }
                }
                else {
                    // suntem intr-un calup diferit de ultimul, il vom umple cu biti de 1 incepand
                    // de la maskStartBit pana la sfarsit
                    maskEndBit = CHUNK_SIZE - 1;
                    maskStart = (1 << maskStartBit) - 1;
                    maskEnd = allBitsSet;
                }
                // cum maskStart o sa contina biti de 1 de la inceput si pana la bitul (maskStartBit - 1)
                // daca aplicam operatia XOR intre cele doua masti, vom obtine
                // o masca de biti care contine doar 1 intre bitii de pozitie maskStartBit si maskEndBit
                matrix[i][currentChunk] ^= (maskEnd ^ maskStart);
                currentColumn += maskEndBit - maskStartBit + 1;
            }
        }
    }
}

void query() {
    fin >> numberOfQueries;
    int x, y;
    int chunk, chunkBit;
    while (numberOfQueries--) {
        fin >> x >> y;
        chunk = (y - 1) / CHUNK_SIZE + 1;
        chunkBit = (y - 1) % CHUNK_SIZE;
        if (matrix[x][chunk] & (1 << chunkBit)) {
            fout << 1 << '\n';
        }
        else {
            fout << 0 << '\n';
        }
    }
}

int main()
{
    read();
    update();
    query();

    fin.close();
    fout.close();
    return 0;
}
